@@ -1,5 +1,7 @@
+import time
 from xml.etree.ElementTree import Element, SubElement
 from elifearticle import utils as eautils
+from elifetools import utils as etoolsutils
 from jatsgenerator import utils
 
 
@@ -114,11 +116,12 @@ def set_contrib_role(parent, contrib_type, contributor=None):
 
 def set_contrib_orcid(parent, contributor):
     if contributor.orcid:
+        orcid_value = etoolsutils.orcid_uri_to_orcid(contributor.orcid)
         orcid_tag = SubElement(parent, "contrib-id")
         if contributor.orcid_authenticated:
             orcid_tag.set("authenticated", "true")
         orcid_tag.set("contrib-id-type", "orcid")
-        orcid_tag.text = "http://orcid.org/" + contributor.orcid
+        orcid_tag.text = "http://orcid.org/" + orcid_value
 
 
 def set_contrib_corresp(parent, rid):
@@ -152,6 +155,16 @@ def set_aff(
     if aff_id:
         aff.set("id", aff_id)
 
+    # if there is text and no other essential attributes, set the text and return
+    essential_attributes = ["institution"]
+    if (
+        affiliation.text
+        and len([name for name in essential_attributes if getattr(affiliation, name)])
+        <= 0
+    ):
+        aff.text = affiliation.text
+        return
+
     if institution_wrap:
         institution_parent_tag = SubElement(aff, "institution-wrap")
     else:
@@ -261,6 +274,55 @@ def set_history(parent, poa_article, date_types):
             set_date(history, poa_article, date_type)
 
 
+def set_publication_history(parent, poa_article):
+    pub_history = SubElement(parent, "pub-history")
+    for event in poa_article.publication_history:
+        event_tag = SubElement(pub_history, "event")
+        if event.event_desc:
+            event_desc_tag = SubElement(event_tag, "event-desc")
+            event_desc_tag.text = event.event_desc
+        if event.date:
+            event_date_tag = SubElement(event_tag, "date")
+            event_date_tag.set("date-type", event.event_type)
+            event_date_tag.set("iso-8601-date", time.strftime("%Y-%m-%d", event.date))
+            set_dmy(event_date_tag, event.date)
+        if event.uri:
+            self_uri_tag = SubElement(event_tag, "self-uri")
+            self_uri_tag.set("content-type", event.event_type)
+            self_uri_tag.set("xlink:href", event.uri)
+
+
+def set_sub_articles(parent, article):
+    for review_article in article.review_articles:
+        set_sub_article(parent, review_article)
+
+
+def set_sub_article(parent, article):
+    sub_article_tag = SubElement(parent, "sub-article")
+    if article.id:
+        sub_article_tag.set("id", article.id)
+    if article.article_type:
+        sub_article_tag.set("article-type", article.article_type)
+    front_stub_tag = SubElement(sub_article_tag, "front-stub")
+    set_article_id(front_stub_tag, article)
+    set_title_group(front_stub_tag, article)
+    if article.contributors:
+        for contributor in article.contributors:
+            contrib_group_tag = SubElement(front_stub_tag, "contrib-group")
+            contrib_tag = SubElement(contrib_group_tag, "contrib")
+            contrib_tag.set("contrib-type", contributor.contrib_type)
+            set_contrib_name(contrib_tag, contributor)
+            set_contrib_orcid(contrib_tag, contributor)
+            # add inline aff tags
+            for affiliation in contributor.affiliations:
+                set_aff(
+                    contrib_tag,
+                    affiliation,
+                    contributor.contrib_type,
+                    institution_wrap=True,
+                )
+
+
 def set_license(parent, poa_article):
     license_tag = SubElement(parent, "license")
 
@@ -331,8 +393,19 @@ def set_abstract(parent, poa_article):
     """
     root_tag_name = "abstract"
     tag_name = "p"
+
+    # do not double-wrap in a p tag
+    if poa_article.abstract.startswith("<p>"):
+        # note: multiple p tag content will be collapsed into one p tag for now
+        abstract_text = etoolsutils.remove_tag("p", poa_article.abstract)
+    else:
+        abstract_text = poa_article.abstract
+
     root_xml_element = Element(root_tag_name)
-    new_tag = utils.append_to_tag(root_xml_element, tag_name, poa_article.abstract)
+    new_tag = utils.append_to_tag(
+        root_xml_element, tag_name, abstract_text, utils.XML_NAMESPACE_MAP
+    )
+
     parent.append(new_tag)
 
 
@@ -353,6 +426,10 @@ def get_contrib_par_ids(poa_article, auth_id):
 def compare_aff(aff1, aff2):
     # Compare two affiliations by comparing the object attributes
     attrs = ["city", "country", "department", "institution"]
+    # if all attributes are empty, check the text attribute
+    if len([name for name in attrs if getattr(aff1, name)]) <= 0:
+        attrs = ["text"]
+
     for attr in attrs:
         if (
             getattr(aff1, attr)
@@ -448,9 +525,10 @@ def set_award_group(parent, award, par_id):
     award_group.set("id", par_id)
     if award.institution_name or award.institution_id:
         set_funding_source(award_group, award.institution_id, award.institution_name)
-    for award_id in award.award_ids:
-        award_id_tag = SubElement(award_group, "award-id")
-        award_id_tag.text = award_id
+    for award_object in award.awards:
+        if award_object.award_id:
+            award_id_tag = SubElement(award_group, "award-id")
+            award_id_tag.text = award_object.award_id
     if award.principal_award_recipients:
         set_principal_award_recipients(award_group, award)
 
@@ -483,11 +561,11 @@ def set_pub_date(parent, poa_article, pub_type):
     # pub-date pub-type = pub_type
     date = poa_article.get_date(pub_type)
     if date:
-        if pub_type == "pub":
+        if pub_type in ["posted_date", "pub"]:
             date_tag = SubElement(parent, "pub-date")
             date_tag.set("date-type", pub_type)
             date_tag.set("publication-format", "electronic")
-            set_dmy(date_tag, date)
+            set_dmy(date_tag, date.date)
 
 
 def set_date(parent, poa_article, date_type):
@@ -496,16 +574,16 @@ def set_date(parent, poa_article, date_type):
     if date:
         date_tag = SubElement(parent, "date")
         date_tag.set("date-type", date_type)
-        set_dmy(date_tag, date)
+        set_dmy(date_tag, date.date)
 
 
 def set_dmy(parent, date):
     day = SubElement(parent, "day")
-    day.text = str(date.date.tm_mday).zfill(2)
+    day.text = str(date.tm_mday).zfill(2)
     month = SubElement(parent, "month")
-    month.text = str(date.date.tm_mon).zfill(2)
+    month.text = str(date.tm_mon).zfill(2)
     year = SubElement(parent, "year")
-    year.text = str(date.date.tm_year)
+    year.text = str(date.tm_year)
 
 
 def set_author_notes(parent, poa_article):
@@ -526,7 +604,14 @@ def set_corresp(parent, contributor, corresp_count):
         if affiliation.email:
             email = affiliation.email
             break
-    initials = "" + contributor.given_name[0] + contributor.surname[0]
+    # concatenate initials for the contributor
+    initials = "".join(
+        [
+            name_part[0]
+            for name_part in [contributor.given_name, contributor.surname]
+            if name_part
+        ]
+    )
     if email:
         corresp = SubElement(parent, "corresp")
         corresp.set("id", "cor" + str(corresp_count))
